﻿/****************************** Module Header ******************************\
Module Name:  MainForm
Project:      TraceCapture
Copyright (c) 2019 WTG

This class demonstrates how to find, open, and communicate with
usb peak power meters to display trace information and change settings such as timebase
and video bandwidth.


THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
\***************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using PowerMeterManagement;
using PwrSnsrLib;

namespace TraceCapture
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// Constructor for this class.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
            //Initialize meter communicator to closed.
            MeterOpen = false;
            //Find any resources currently connected.
            FindResources();
            chart1.GetToolTipText += chart1_GetToolTipText;

        }

        /// <summary>
        /// Event handler for chart tool tips
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void chart1_GetToolTipText(object sender, System.Windows.Forms.DataVisualization.Charting.ToolTipEventArgs e)
        {
            int index = e.HitTestResult.PointIndex;

            if (index > -1)
            {
                var series = e.HitTestResult.Series;
                if (series != null)
                {
                    var xval = series.Points[index].XValue;
                    var yval = series.Points[index].YValues[0];

                    e.Text = Utilities.ToEngineeringNotation(xval, 4) + "s, " +
                        yval.ToString("F3") + " dBm";

                }
                else
                {

                }
            }
        }
        /// <summary>
        /// Object used to access the IVI-COM driver
        /// </summary>
        int MeterHandle { set; get; }

        /// <summary>
        /// Finds all available resources when button is clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            try
            {
                FindResources();
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message,"Error", MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Uses the Boonton55xxxClass to find available resources.
        /// </summary>
        void FindResources()
        {
            //Clear current list.
            comboBox1.Items.Clear();
            //Find resources and build a pip-delimited string.
            int buffSize = 256;
            byte[] buff = new byte[buffSize];
            int err = PwrSnsrLib32.PwrSnsr_FindResources("|", buffSize, buff);
            Utilities.ThrowifError(err);
            var s = Utilities.DecodeAscii(buff);
            //Check if string is valid.
            if (!string.IsNullOrWhiteSpace(s))
            {
                //Split string into array.
                var resources = s.Split('|');
                //Loop throught the results and add them to the combobox
                foreach (string resource in resources)
                {
                    comboBox1.Items.Add(resource);
                }
            }

            //Select first item if combobox is not empty.
            if (comboBox1.Items.Count > 0)
            {
                comboBox1.SelectedIndex = 0;
            }
           
        }

        /// <summary>
        /// Fetches a waveform from the meter and displays it on a chart.
        /// </summary>
        void DisplayTrace()
        {
            //If the m_meter is not open, return.
            if (!MeterOpen)
            {
                return;
            }
           
            

            //Fetch the waveform
            int traceCount = 501;
            string chan = "CH1";
            float[] trace = new float[traceCount];
            int actualSize = 0;
            int ret = PwrSnsrLib32.PwrSnsr_FetchWaveform(MeterHandle, chan, traceCount, trace, ref actualSize);
            Utilities.ThrowifError(ret);            
            //Cast the data to a float array
            var data = trace.Cast<float>().ToArray();
            //Get the timebase.
            float timebase = 0.0f;
            ret = PwrSnsrLib32.PwrSnsr_GetTimebase(MeterHandle, ref timebase);
            Utilities.ThrowifError(ret);
            //Get the trig position
            float position = 0.0f;
            ret = PwrSnsrLib32.PwrSnsr_GetTrigVernier(MeterHandle, ref position);
            Utilities.ThrowifError(ret);
            //Get trigger delay
            float delay = 0.0f;         
            ret = PwrSnsrLib32.PwrSnsr_GetTrigDelay(MeterHandle, ref delay);
            Utilities.ThrowifError(ret);
            //Get the start of the trace.
            float start = (-(position * timebase) + delay);


            //Clear all data points.
            chart1.Series[0].Points.Clear();
            //Add each point to the chart data series. X = time, y = power.
            for (int i = 0; i < data.Length; i++)
            {
                chart1.Series[0].Points.AddXY(start + ((i * timebase * 10f) / ((float)data.Length)), data[i]);
            }           
        }
        /// <summary>
        /// Displays a single trace when button is pushed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button2_Click(object sender, EventArgs e)
        {
            try
            {
                DisplayTrace();
            
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,"Error",  MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
        }

        /// <summary>
        /// Used to monitor toggling of open/close button.
        /// </summary>
        bool MeterOpen
        {
            set;
            get;
        }

        /// <summary>
        /// Opens a session to the sensor
        /// </summary>
        void OpenSensor()
        {
            MeterOpen = false;
            //Initilize driver using the resource name in combobox1.            
            int handle = 0;
            var ret = PwrSnsrLib32.PwrSnsr_init(comboBox1.Text, ref handle);
            Utilities.ThrowifError(ret);
            //Set to defaults
            ret = PwrSnsrLib32.PwrSnsr_reset(handle);
            Utilities.ThrowifError(ret);

            MeterHandle = handle;
            MeterOpen = true;

            //Set for continuous run mode.
            ret = PwrSnsrLib32.PwrSnsr_SetInitiateContinuous(MeterHandle, 1);
            Utilities.ThrowifError(ret);

            //Get the maximum timebase allowed on this sensor.
            float max = 0.0f;
            ret = PwrSnsrLib32.PwrSnsr_GetMaxTimebase(MeterHandle, ref max);
            Utilities.ThrowifError(ret);
            //Construct max label.
            string maxLabel = Utilities.GetLabelFromTimebase(max);
            //Find the index of the max timebase
            int index = Array.IndexOf(Utilities.TimebaseLabels, maxLabel); 
            //Clear the old values, if any.
            comboBox2.Items.Clear();
            //Add the relevant timebase labels.
            for (int i = 0; i <= index; i++)
            {
                comboBox2.Items.Add(Utilities.TimebaseLabels[i]);
            }
            //Get the current timebase and label.
            float timebase = 0.0f;
            ret = PwrSnsrLib32.PwrSnsr_GetTimebase(MeterHandle, ref timebase);
            Utilities.ThrowifError(ret);
            string currentTimebase = Utilities.GetLabelFromTimebase(timebase);
            //Set the selected item to the current timebase.
            comboBox2.SelectedItem = currentTimebase;
            //Set the bandwitch button state.
            PwrSnsrBandwidthEnum bw = PwrSnsrBandwidthEnum.BandwidthHigh;
            ret = PwrSnsrLib32.PwrSnsr_GetBandwidth(MeterHandle,"CH1", ref bw);
            Utilities.ThrowifError(ret);
            checkBox2.Checked = (bw == PwrSnsrBandwidthEnum.BandwidthLow);            
            //Set averaging to 8 vis comboBox3
            comboBox3.SelectedIndex = 3;
            
        }

        /// <summary>
        /// Opens or closes the session to the meter.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button3_Click(object sender, EventArgs e)
        {
            
            try
            {
                //Open meter if closed or vice versa.
                if (!MeterOpen)
                {
                    //Open the session.
                    OpenSensor();                   
                    button3.Text = "Close";
                    MeterOpen = true;
                    //Set for continuous capturing by checking the checkbox.
                    checkBox1.Checked = true;
                }
                else
                {
                    //Close meter.                   
                    PwrSnsrLib32.PwrSnsr_close(MeterHandle);                    
                    MeterOpen = false;
                    button3.Text = "Open";
                }                
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,"Error",  MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
        }
        /// <summary>
        /// Used to provide exlusive access to the timer event.
        /// </summary>
        Mutex m_mutex = new Mutex();
        /// <summary>
        /// Time event for displaying traces.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            try
            {
                //Try to get the mutex for 5 milliseconds. If attempt fails, move on so we
                //don't accumulate timer events.
                if (m_mutex.WaitOne(5))
                {
                    //Display a new trace.
                    DisplayTrace();
                }
            }
            catch
            {
            }
            try
            {
                //release the mutex.
                m_mutex.ReleaseMutex();
            }
            catch
            {                
            }
        }
        /// <summary>
        /// Enables or disables the timer based on the checkbox state.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            timer1.Enabled = checkBox1.Checked;
        }
        /// <summary>
        /// Event changes timebase based on the selection.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            
            try
            {
                if (!string.IsNullOrWhiteSpace(comboBox2.Text))
                {
                    //Parse the current text in the combobox and set the meter's timebase.
                    float timebase = (float)Utilities.GetTimebaseFromLabel(comboBox2.Text);
                    int err = PwrSnsrLib32.PwrSnsr_SetTimebase(MeterHandle, timebase);
                    Utilities.ThrowifError(err);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
        /// <summary>
        /// Selects high or low bandwidth based on the checkbox's checked state.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            try
            {
                //Select the bandwidth.
                PwrSnsrBandwidthEnum bw = checkBox2.Checked ? PwrSnsrBandwidthEnum.BandwidthLow
                    : PwrSnsrBandwidthEnum.BandwidthHigh;
                //Set the bandwidth.                
                int err = PwrSnsrLib32.PwrSnsr_SetBandwidth(MeterHandle,"CH1", bw);
                Utilities.ThrowifError(err);
                //Set the button text.
                checkBox2.Text = checkBox2.Checked ? "Low " : "High";
               
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Set the meter averaging based on the combobox selection
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void comboBox3_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {               

                int avging = int.Parse(comboBox3.Text);
                int err = PwrSnsrLib32.PwrSnsr_SetAverage(MeterHandle, "CH1", avging);
                Utilities.ThrowifError(err);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //Clear the trace data.
        private void button4_Click(object sender, EventArgs e)
        {
            try
            {
                chart1.Series[0].Points.Clear();
                if (MeterOpen)
                {                   
                    int err = PwrSnsrLib32.PwrSnsr_Clear(MeterHandle);
                    Utilities.ThrowifError(err);
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //Set trace width based on window size.
        private void chart1_Resize(object sender, EventArgs e)
        {
            chart1.Series[0].BorderWidth = Math.Max(1,chart1.Width / 500);
        }

      
    }
}
